"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.collectAPIMetrics = void 0;
const onFinished = require("on-finished");
const describe_1 = require("./describe");
const collectAPIMetrics = (metrics) => {
    (0, describe_1.describeAPIMetricsOnce)(metrics);
    const modifyReq = (req) => {
        req._metrics_gatherer = {
            labels: {},
        };
        if (!req.connection._metrics_gatherer) {
            req.connection._metrics_gatherer = {};
        }
    };
    const observeBytesRW = (req) => {
        const bytesReadPreviously = req.connection._metrics_gatherer.bytesRead || 0;
        const bytesWrittenPreviously = req.connection._metrics_gatherer.bytesWritten || 0;
        return () => {
            const bytesReadDelta = req.connection.bytesRead - bytesReadPreviously;
            const bytesWrittenDelta = req.connection.bytesWritten - bytesWrittenPreviously;
            req.connection._metrics_gatherer.bytesRead = req.connection.bytesRead;
            req.connection._metrics_gatherer.bytesWritten =
                req.connection.bytesWritten;
            metrics.histogram('api_bytes_read', bytesReadDelta, req._metrics_gatherer.labels);
            metrics.histogram('api_bytes_written', bytesWrittenDelta, req._metrics_gatherer.labels);
        };
    };
    const observeLatency = (req) => {
        const t0 = process.hrtime();
        return () => {
            const dt = process.hrtime(t0);
            const duration = dt[0] * 1000 + dt[1] / 1e6;
            metrics.histogram('api_latency_milliseconds', duration, req._metrics_gatherer.labels);
        };
    };
    return (req, res, next) => {
        modifyReq(req);
        metrics.counter('api_arrival_total', 1, req._metrics_gatherer.labels);
        const onFinishFuncs = [observeBytesRW(req), observeLatency(req)];
        onFinished(res, () => {
            req._metrics_gatherer.labels.state = req.aborted
                ? 'aborted'
                : 'completed';
            req._metrics_gatherer.labels.statusCode = res.statusCode || '';
            onFinishFuncs.forEach((f) => f());
        });
        next();
    };
};
exports.collectAPIMetrics = collectAPIMetrics;
//# sourceMappingURL=collect.js.map